package com.hz.pm.api.projectdeclared.controller;

import com.hz.pm.api.common.util.ExcelDownUtil;
import com.hz.pm.api.projectdeclared.manage.ConstructionManage;
import com.hz.pm.api.projectdeclared.model.dto.PaymentPlanSupplementDTO;
import com.hz.pm.api.projectdeclared.model.dto.PreInsSaveDTO;
import com.hz.pm.api.projectdeclared.model.req.XcfhxApplyReq;
import com.hz.pm.api.projectdeclared.model.vo.ContractVO;
import com.hz.pm.api.projectdeclared.model.vo.PreInsVO;
import com.hz.pm.api.projectdeclared.model.vo.ProjectContractListVO;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * @Classname ConstructionController
 * @Description 申报项目-建设中
 * @Date 2023/5/29 09:29
 * @Author PoffyZhang
 */
@Slf4j
@Validated
@RestController
@RequestMapping("/api/v1/declared/construction")
@Api(value = "ConstructionController", tags = "申报管理-建设中")
@RequiredArgsConstructor
public class ConstructionController {
    private final ConstructionManage constructionManage;

    @ApiOperation(value = "待合同备案的项目列表", notes = "合同备案的项目列表")
    @GetMapping("/contract/project-list")
    public PageVo<ProjectLibListItemVO> projectlist(@ModelAttribute ProjectListReq req) {
        return constructionManage.projectLibList(req);
    }

    @ApiOperation(value = "已经完善合同信息的项目列表", notes = "已经完善合同信息的项目列表")
    @GetMapping("/contract/project-list-perfect")
    public PageVo<ProjectContractListVO> projectlistPerfect(@ModelAttribute ProjectListReq req) {
        return constructionManage.projectListPerfect(req);
    }

    @GetMapping("/contract/export")
    @ApiOperation("待合同备案的项目列表导出")
    @WebLog("待合同备案的项目列表导出")
    public void exportList(ProjectListReq req, HttpServletResponse response) {
        ExcelDownUtil.downXls(response, req, constructionManage::exportList);
    }

    @ApiOperation(value = "合同备案的详情")
    @GetMapping("/contract/detail/{projectId}")
    public List<ContractVO> detailContractByProjectId(@PathVariable Long projectId) {
        return constructionManage.detailContractByProjectId(projectId);
    }

    @ApiOperation(value = "补充实际支付金额", notes = "补充实际支付金额")
    @WebLog("补充实际支付金额")
    @PostMapping("/supplement")
    public String supplement(@Validated @RequestBody List<PaymentPlanSupplementDTO> plans) {
        return constructionManage.supplement(plans);
    }

    @ApiOperation(value = "待初验备案的项目列表")
    @GetMapping("/pre-ins/project-list")
    public PageVo<ProjectLibListItemVO> preProjectList(@ModelAttribute ProjectListReq req) {
        return constructionManage.preProjectList(req);
    }

    @GetMapping("/pre-ins/export")
    @ApiOperation("待初验备案的项目列表导出")
    @WebLog("待初验备案的项目列表导出")
    public void exportPreList(ProjectListReq req, HttpServletResponse response) {
        ExcelDownUtil.downXls(response, req, constructionManage::exportPreList);
    }

    @ApiOperation(value = "初验备案的详情-通过项目ID", notes = "初验备案的详情-通过项目ID")
    @GetMapping("/pre-ins/detail/{projectId}")
    public List<PreInsVO> detailPreInsByProjectId(@PathVariable Long projectId) {
        return constructionManage.detailPreInsByProjectId(projectId);
    }

    @ApiOperation(value = "上传初验材料")
    @WebLog("上传初验材料")
    @PostMapping("/submit-pre-ins")
    public void submitPreIns(@Validated @RequestBody PreInsSaveDTO dto) {
        constructionManage.submitFirstInspectedFiles(dto);
    }

    @ApiOperation("信创符合性申请")
    @WebLog("信创符合性申请")
    @PostMapping("/submitXcfhxApply")
    public void submitXcfhxApply(@RequestBody @Valid XcfhxApplyReq req) {
        constructionManage.submitXcfhxApply(req);
    }

    @ApiOperation("信创符合性申请-手动新增的")
    @WebLog("信创符合性申请-手动新增的")
    @PostMapping("/submitXcfhxApply-manual/{id}")
    public String submitXcfhxApplyManual(@PathVariable Long id) {
        return constructionManage.submitXcfhxApplyManual(id);
    }
}
