package com.hz.pm.api.projectdeclared.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.statemachine.event.AbstractStateChangeEvent;
import com.hz.pm.api.common.statemachine.event.AdaptStateChangeEvent;
import com.hz.pm.api.common.statemachine.event.SelfTestStateChangeEvent;
import com.hz.pm.api.common.statemachine.event.TestValidStateChangeEvent;
import com.hz.pm.api.common.statemachine.util.*;
import com.hz.pm.api.common.util.*;
import com.hz.pm.api.external.MhApiClient;
import com.hz.pm.api.external.MhFileClient;
import com.hz.pm.api.external.model.dto.MhPurchaseIntentionDTO;
import com.hz.pm.api.external.model.dto.MhPurchaseNoticeDTO;
import com.hz.pm.api.projectdeclared.helper.MhXchxFileHelper;
import com.hz.pm.api.projectdeclared.model.dto.DeclaredProjectExportDTO;
import com.hz.pm.api.projectdeclared.model.dto.PurchaseSaveDTO;
import com.hz.pm.api.projectdeclared.model.dto.XcfhxApplyEditDTO;
import com.hz.pm.api.projectdeclared.model.dto.XcfhxApplySaveDTO;
import com.hz.pm.api.projectdeclared.model.entity.*;
import com.hz.pm.api.projectdeclared.model.enumerization.BidTypeEnum;
import com.hz.pm.api.projectdeclared.model.enumerization.PurchaseScopeEnum;
import com.hz.pm.api.projectdeclared.model.enumerization.PurchaseSourceTypeEnum;
import com.hz.pm.api.projectdeclared.model.req.*;
import com.hz.pm.api.projectdeclared.model.req.SubmitPurchaseNoticeReq.PurchaseNoticeTenderDTO;
import com.hz.pm.api.projectdeclared.model.vo.PurchaseVO;
import com.hz.pm.api.projectdeclared.model.vo.TestValidListVO;
import com.hz.pm.api.projectdeclared.model.vo.XcfhxApplyListVO;
import com.hz.pm.api.projectdeclared.model.vo.XinchuangVO;
import com.hz.pm.api.projectdeclared.service.*;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.helper.ProjectManageUtil;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.enumeration.ProjectTypeNewEnum;
import com.hz.pm.api.projectlib.model.enumeration.status.*;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.model.vo.TenderListInfoVO;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import com.hz.pm.api.sys.manage.ProcessModelManage;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.NdDateUtils;
import com.ningdatech.file.entity.File;
import com.ningdatech.file.entity.vo.result.FileResultVO;
import com.ningdatech.file.service.FileService;
import com.wflow.bean.entity.WflowModels;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * PurchaseManage
 * </p>
 *
 * @author WendyYang
 * @since 11:15 2024/4/2
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class PurchaseManage {

    private final IProjectService projectService;
    private final UserInfoHelper userInfoHelper;
    private final IContractService contractService;
    private final IPurchaseService purchaseService;
    private final TenderStateMachineUtil tenderStateMachineUtil;
    private final ProjectStateMachineUtil projectStateMachineUtil;
    private final MhApiClient mhApiClient;
    private final MhFileClient mhFileClient;
    private final FileService fileService;
    private final IOperationService operationService;
    private final ProcessModelManage processModelManage;
    private final DeclaredProjectHelper declaredProjectManage;
    private final ProcessInstanceService processInstanceService;
    private final TestValidStateMachineUtil testValidStateMachineUtil;
    private final SelfTestStateMachineUtil selfTestStateMachineUtil;
    private final AdaptStateMachineUtil adaptStateMachineUtil;
    private final IPurchaseInstService purchaseInstService;
    private final IProjectInstService projectInstService;
    private final EnvironmentUtil environmentUtil;
    private final IPurchaseStatusChangeService purchaseStatusChangeService;

    private final IXinchuangService xinchuangService;

    private final MhXchxFileHelper mhXchxFileHelper;

    /**
     * 待采购的-项目列表
     *
     * @param req \
     * @return \
     */
    public PageVo<ProjectLibListItemVO> projectLibList(ProjectListReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        Assert.notNull(user, "获取登录用户失败!");
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(req);
        //只能看自己单位的
        query.eq(Project::getBuildOrgCode, user.getMhUnitIdStr());
        //待采购状态
        query.in(Project::getStatus, Lists.newArrayList(ProjectStatus.TO_BE_PURCHASED.getCode(),
                ProjectStatus.ON_PURCHASING.getCode()));
        query.eq(Project::getStage, ProjectStatus.PROJECT_APPROVED.getCode());
        query.eq(Project::getNewest, Boolean.TRUE);
        query.orderByAsc(Project::getApprovalDate);
        Page<Project> page = projectService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<Long> projectIds = CollUtils.fieldList(page.getRecords(), Project::getId);
        List<Purchase> purchases = purchaseService.listByProjectIds(projectIds);
        Map<Long, List<Purchase>> purchaseMap = CollUtils.group(purchases, Purchase::getProjectId);
        Map<Long, Contract> contractMap = contractService.listByProjectIds(projectIds);
        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setProjectName(w.getProjectName());
            item.setCreateOn(w.getCreateOn());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectType(w.getProjectType());
            item.setProjectYear(w.getProjectYear());
            item.setBuildOrg(w.getBuildOrgName());
            item.setBizDomain(w.getBizDomain());
            item.setProcessStatus(w.getProcessStatus());
            item.setInstCode(w.getInstCode());
            item.setIsHigherSuperOrg(w.getIsHigherSuperOrg());
            item.setApprovedAmount(w.getApprovalAmount());
            item.setReviewAmount(w.getReviewAmount());
            item.setApprovalDate(w.getApprovalDate());
            List<Purchase> currPurchases = purchaseMap.get(w.getId());
            if (currPurchases != null) {
                item.setTenders(CollUtils.convert(currPurchases, x -> {
                    TenderListInfoVO tender = new TenderListInfoVO();
                    tender.setBidName(x.getBidName());
                    tender.setBidId(x.getId());
                    tender.setBidStatus(x.getStatus());
                    tender.setBidStatusName(TenderStatus.getDescByStatus(x.getStatus()));
                    Contract contract = contractMap.get(x.getId());
                    if (contract != null) {
                        tender.setConstructionAmount(contract.getTotalAmount());
                    }
                    return tender;
                }));
            }
            return item;
        });
        return PageVo.of(records, page.getTotal());
    }

    public void exportList(HttpServletResponse response, ProjectListReq param) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        VUtils.isTrue(Objects.isNull(user)).throwMessage("获取登录用户失败!");
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(param);
        //待采购状态
        //只能看自己单位的
        query.eq(Project::getBuildOrgCode, user.getMhUnitId());
        query.eq(Project::getStatus, ProjectStatus.TO_BE_PURCHASED.getCode());
        query.eq(Project::getStage, ProjectStatus.PROJECT_APPROVED.getCode());
        query.eq(Project::getNewest, Boolean.TRUE);
        query.orderByAsc(Project::getApprovalDate);
        List<Project> records = projectService.list(query);
        Assert.notEmpty(records, "暂无数据可导出");
        List<DeclaredProjectExportDTO> data = Lists.newArrayList();

        AtomicInteger serialNumber = new AtomicInteger(0);
        records.forEach(r -> {
            DeclaredProjectExportDTO dpe = new DeclaredProjectExportDTO();
            BeanUtils.copyProperties(r, dpe);
            dpe.setProjectTypeName(ProjectTypeNewEnum.getDesc(r.getProjectType()));
            dpe.setStatusName(ProjectStatus.getDesc(r.getStatus()));
            String createOnStr = NdDateUtils.format(r.getCreateOn(), "yyyy-MM-dd HH:mm");
            dpe.setCreateOn(createOnStr);
            dpe.setSerialNumber(serialNumber.incrementAndGet());
            dpe.setApprovedAmount(r.getApprovalAmount());
            dpe.setApprovalDate(r.getApprovalDate().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
            data.add(dpe);
        });

        String fileName = "采购结果待备案项目列表";
        ExcelDownUtil.setFileName(fileName, response);
        //数据导出处理函数
        try {
            EasyExcel.write(response.getOutputStream(), DeclaredProjectExportDTO.class)
                    .autoCloseStream(false)
                    .registerWriteHandler(ExcelExportStyle.formalStyle())
                    .sheet(fileName)
                    .doWrite(data);
        } catch (IOException e) {
            log.error("导出异常", e);
            throw BizException.wrap("导出失败，请重试");
        }
    }

    /**
     * 获取采购结果备案详情
     *
     * @param projectId
     * @return
     */
    public List<PurchaseVO> detailByProjectId(Long projectId) {
        List<Purchase> purchases = purchaseService.listByProjectId(projectId);
        return BeanUtil.copyToList(purchases, PurchaseVO.class);
    }

    /**
     * 填写采购结果
     *
     * @param dto
     * @return
     */
    public String submitResult(PurchaseSaveDTO dto) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        Long projectId = dto.getProjectId();
        VUtils.isTrue(Objects.isNull(projectId)).throwMessage("提交失败 缺少项目ID!");
        Project project = projectService.getNewestNoNull(dto.getProjectId());

        //首先要判断 项目当前状态 是不是 采购结果备案
        VUtils.isTrue(!ProjectStatus.TO_BE_PURCHASED.eq(project.getStatus())
                        || !ProjectStatus.PROJECT_APPROVED.eq(project.getStage()))
                .throwMessage("提交失败 该项目不是 待采购备案状态或者已立项阶段");

        List<Purchase> purchases = purchaseService.listByProjectId(projectId);
        Map<Long, Purchase> purchaseMap = CollUtils.listToMap(purchases, Purchase::getId);
        List<Purchase> purchaseEntities = CollUtils.convert(dto.getTenders(), w -> {
            Purchase purchase = BeanUtil.copyProperties(w, Purchase.class);
            purchase.setProjectId(projectId);
            if (!purchaseMap.containsKey(w.getId())) {
                purchase.setCreateBy(user.getUserIdStr());
            }
            return purchase;
        });

        if (purchaseService.saveOrUpdateBatch(purchaseEntities)) {
            projectStateMachineUtil.pass(project);
            project.setUpdateOn(LocalDateTime.now());

            // 获取总的成交时间及金额
            LocalDateTime lastTransactionTime = null;
            BigDecimal transactionAmountTotal = BigDecimal.ZERO;
            for (Purchase purchase : purchaseEntities) {
                if (lastTransactionTime == null || purchase.getTransactionTime().isAfter(lastTransactionTime)) {
                    lastTransactionTime = purchase.getTransactionTime();
                }
                transactionAmountTotal = transactionAmountTotal.add(purchase.getTransactionAmount());
            }

            project.setTransactionAmount(transactionAmountTotal);
            project.setTransactionTime(lastTransactionTime);
            projectService.updateById(project);

            return "填写成功";
        }

        return "保存失败";
    }

    @Value("${mh.purchase-notice.open:true}")
    private Boolean mhPurchaseNoticeOpen;

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitPurchaseNotice(SubmitPurchaseNoticeReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        Long projectId = req.getProjectId();
        Project project = projectService.getNewestNoNull(projectId);

        //首先要判断 项目当前状态 是不是 采购结果备案
        VUtils.isTrue(!ProjectStatus.TO_BE_PURCHASED.eq(project.getStatus())
                        || !ProjectStatus.PROJECT_APPROVED.eq(project.getStage()))
                .throwMessage("提交失败,该项目不是待采购备案状态或者已立项阶段");
        List<MhPurchaseNoticeDTO> notices = new ArrayList<>();
        List<Purchase> purchases = req.getTenders().stream().map(w -> {
            Purchase purchase = new Purchase();
            purchase.setPurchaseMethod(w.getPurchaseMethod());
            purchase.setBidAddress(w.getBidAddress());
            purchase.setBidName(w.getBidName());
            purchase.setBidBudget(w.getBidBudget());
            purchase.setBidDoc(w.getBidDoc());
            purchase.setBidStartTime(w.getBidStartTime());
            purchase.setBidEndTime(w.getBidEndTime());
            purchase.setId(w.getBidId());
            purchase.setProjectId(projectId);
            purchase.setPurchaseConnectAddress(w.getPurchaseConnectAddress());
            purchase.setPurchasePhone(w.getPurchasePhone());
            purchase.setPurchaseRemark(w.getPurchaseRemark());
            purchase.setPurchaseScope(w.getPurchaseScope());
            purchase.setPurchaseUnit(w.getPurchaseUnit());
            purchase.setPurchaseUnitId(w.getPurchaseUnitId());
            purchase.setPurchaseContact(w.getPurchaseContact());
            purchase.setBidType(w.getBidType());
            if (purchase.getId() == null) {
                purchase.setCreateBy(user.getUserIdStr());
            }
            purchase.setSelfTestStatus(TenderSelfTestStatus.WITHOUT_SELF_TEST_INFO.getCode());
            purchase.setTestValidStatus(TenderTestValidStatus.WITHOUT_TEST_VALID_INFO.getCode());
            purchase.setAdaptStatus(TenderAdaptStatus.WITHOUT_ADAPT_INFO.getCode());
            purchase.setStatus(TenderStatus.TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO.getCode());
            purchase.setXcfhxApplyStatus(TenderXcfhxApplyStatus.TO_BE_SUBMIT_XCFHX_APPLY.getCode());
            if (Boolean.TRUE.equals(mhPurchaseNoticeOpen)) {
                notices.add(buildPurchaseNotice(w, user));
            }
            return purchase;
        }).collect(Collectors.toList());
        purchaseService.saveOrUpdateBatch(purchases);
        projectStateMachineUtil.pass(project);
        projectService.updateById(project);
        // 发布采购公告
        if (Boolean.TRUE.equals(mhPurchaseNoticeOpen)) {
            mhApiClient.publishPurchaseNotice(notices);
        }
    }

    private MhPurchaseNoticeDTO buildPurchaseNotice(PurchaseNoticeTenderDTO w, UserInfoDetails user) {
        if (environmentUtil.isDevEnv()) {
            return null;
        }
        Date now = new Date();
        List<FileResultVO> bidFiles = JSONUtil.toList(w.getBidDoc(), FileResultVO.class);
        List<File> files = fileService.listByIds(CollUtils.fieldSet(bidFiles, FileResultVO::getId));
        String fileIds = files.stream().map(file -> {
            String fileUrl = fileService.findUrlByFile(file);
            String filePath = FileUtil.getTmpDirPath() + file.getOriginalFileName();
            java.io.File tmpFile = new java.io.File(filePath);
            HttpUtil.downloadFile(fileUrl, tmpFile);
            String fileId = mhFileClient.uploadFile(tmpFile);
            tmpFile.deleteOnExit();
            return fileId;
        }).collect(Collectors.joining(","));
        PurchaseScopeEnum scope = PurchaseScopeEnum.getByCode(w.getPurchaseScope())
                .orElseThrow(() -> BizException.wrap("采购范围无效"));
        return MhPurchaseNoticeDTO.builder()
                .bidOpenTime(DateUtil.date(w.getBidStartTime()).toJdkDate())
                .endTime(DateUtil.date(w.getBidEndTime()).toJdkDate())
                .buyDesc(w.getPurchaseRemark())
                .budgetMoney(w.getBidBudget().multiply(BigDecimal.valueOf(10000)))
                .buyItem(w.getBidName())
                .buyFile(fileIds)
                .buyScope(scope.getMhCode())
                .buyWay(w.getPurchaseMethod())
                .buyUnitName(w.getPurchaseUnit())
                .createUnit(w.getPurchaseUnitId().intValue())
                .createTime(now)
                .updateTime(now)
                .createUser(user.getMhUserId())
                .updateUser(user.getMhUserId())
                // 招标采购
                .type("ZBGG")
                .detailAddress(w.getPurchaseConnectAddress())
                .linkAddress(w.getBidAddress())
                .linkUser(w.getPurchaseContact())
                .linkPhone(w.getPurchasePhone())
                .postStatus("Y")
                .build();
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitPurchaseRecord(SubmitPurchaseContractRecordReq req) {
        Purchase purchase = purchaseService.getById(req.getBidId());
        if (!TenderStatus.TO_BE_SUBMIT_PURCHASE_CONSTRUCTION_INFO.eq(purchase.getStatus())) {
            throw BizException.wrap("该标段无法进行采购&合同信息备案");
        }
        // 保存采购信息
        purchase.setId(req.getBidId());
        purchase.setAcceptanceLetter(req.getAcceptanceLetter());
        purchase.setBidBuildContent(req.getBidBuildContent());
        purchase.setBiddingDoc(req.getBiddingDoc());
        purchase.setSupplier(req.getSupplier());
        purchase.setSupplierSocialCreditCode(req.getSupplierSocialCreditCode());
        purchase.setBidType(req.getBidType());
        tenderStateMachineUtil.pass(purchase);
        purchaseService.saveOrUpdate(purchase);

        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        Long projectId = req.getProjectId();
        Project project = projectService.getNewestNoNull(projectId);
        LocalDateTime now = LocalDateTime.now();

        // 保存合同信息
        Contract contract = contractService.getByBidId(req.getBidId());
        if (contract == null) {
            contract = new Contract();
            contract.setCreateBy(user.getUserIdStr());
            contract.setCreateOn(now);
        }
        contract.setContractTime(req.getContractTime());
        contract.setAttachment(req.getAttachment());
        contract.setName(req.getName());
        contract.setProjectId(projectId);
        contract.setProjectCode(project.getProjectCode());
        contract.setBidId(req.getBidId());
        contract.setId(req.getConstructionId());
        contract.setUpdateBy(user.getUserIdStr());
        contract.setUpdateOn(now);
        contract.setTotalAmount(req.getTotalAmount());
        contract.setConstructionNo(req.getConstructionNo());
        contract.setSupplierContact(req.getSupplierContact());
        contract.setSupplierContactInfo(req.getSupplierContactInfo());
        contractService.saveOrUpdate(contract);
        // 修改项目合同金额
        Wrapper<Contract> query = Wrappers.lambdaQuery(Contract.class)
                .eq(Contract::getProjectId, projectId);
        List<Contract> allContracts = contractService.list(query);
        BigDecimal totalAmount = DecimalUtil.sum(allContracts, Contract::getTotalAmount);
        project.setUpdateOn(now);
        project.setContractAmount(totalAmount);
        projectService.updateById(project);
    }

    private PageVo<TestValidListVO> queryData(TestValidListReq req,
                                              SFunction<Purchase, Integer> function,
                                              String actualTimeColumn) {
        LambdaQueryWrapper<Purchase> query = Wrappers.lambdaQuery(Purchase.class)
                .like(StrUtil.isNotBlank(req.getBidName()), Purchase::getBidName, req.getBidName())
                .le(req.getCreateOnMax() != null, Purchase::getCreateOn, req.getCreateOnMax())
                .ge(req.getCreateOnMin() != null, Purchase::getCreateOn, req.getCreateOnMin())
                .eq(req.getStatus() != null, function, req.getStatus())
                .isNotNull(function)
                .orderByDesc(Purchase::getCreateOn);
        boolean display = !Boolean.FALSE.equals(req.getDisplay());
        String existsSql = String.format("select 1 from nd_project_operation no" +
                " where no.bid_id = nd_purchase.id and no.%s is not null", actualTimeColumn);
        if (display) {
            query.and(q1 -> q1.eq(Purchase::getBidType, BidTypeEnum.BUILD_APP.getCode())
                    .or(q2 -> q2.exists(existsSql)));
        } else {
            query.ne(Purchase::getBidType, BidTypeEnum.BUILD_APP.getCode()).notExists(existsSql);
        }
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        if (!purchaseService.buildPurchaseQueryPermission(query, user)) {
            return PageVo.empty();
        }
        Page<Purchase> page = purchaseService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<Purchase> records = page.getRecords();
        Map<Long, LocalDateTime> createOnMap = new HashMap<>();
        if (display) {
            List<Long> bidIds = CollUtils.fieldList(records, Purchase::getId);
            AbstractStateChangeEvent event;
            if ("actual_test_valid_time".equals(actualTimeColumn)) {
                event = TestValidStateChangeEvent.SUBMIT_TEST_VALID_INFO;
            } else if ("actual_adaption_time".equals(actualTimeColumn)) {
                event = AdaptStateChangeEvent.SUBMIT_ADAPT_INFO;
            } else {
                event = SelfTestStateChangeEvent.SUBMIT_SELF_TEST_INFO;
            }
            Wrapper<PurchaseStatusChange> pscQuery = Wrappers.lambdaQuery(PurchaseStatusChange.class)
                    .in(PurchaseStatusChange::getEvent, event.name())
                    .in(PurchaseStatusChange::getBidId, bidIds)
                    .orderByAsc(PurchaseStatusChange::getCreateOn);
            List<PurchaseStatusChange> changes = purchaseStatusChangeService.list(pscQuery);
            Map<Long, PurchaseStatusChange> mapByBidId = BizUtils.groupFirstMap(changes,
                    PurchaseStatusChange::getBidId, Comparator.comparing(PurchaseStatusChange::getCreateOn));
            records.forEach(w -> {
                if (BidTypeEnum.BUILD_APP.getCode().equals(w.getBidType())) {
                    createOnMap.put(w.getId(), w.getCreateOn());
                } else {
                    PurchaseStatusChange change = mapByBidId.get(w.getId());
                    if (change != null) {
                        createOnMap.put(w.getId(), change.getCreateOn());
                    }
                }
            });
        }
        List<Long> projectIds = CollUtils.fieldList(records, Purchase::getProjectId);
        List<Project> projects = projectService.listByIds(projectIds);
        Map<Long, Project> projectMap = CollUtils.listToMap(projects, Project::getId);
        Map<Long, Contract> contractMap = contractService.listByProjectIds(projectIds);
        List<TestValidListVO> data = records.stream().map(w -> {
            TestValidListVO item = new TestValidListVO();
            item.setBidId(w.getId());
            Project project = projectMap.get(w.getProjectId());
            if (project != null) {
                item.setProjectName(project.getProjectName());
            }
            item.setBizName(w.getBidName());
            item.setProjectId(w.getProjectId());
            String projectCode = ProjectIdCodeCacheUtil.get(w.getProjectId());
            item.setProjectCode(projectCode);
            Contract contract = contractMap.get(w.getId());
            if (contract != null) {
                item.setContractAmount(contract.getTotalAmount());
            }
            item.setStatus(function.apply(w));
            if (display) {
                item.setCreateOn(createOnMap.get(w.getId()));
            }
            return item;
        }).collect(Collectors.toList());
        return PageVo.of(data, page.getTotal());
    }

    public PageVo<TestValidListVO> testValidList(TestValidListReq req) {
        return queryData(req, Purchase::getTestValidStatus, "actual_test_valid_time");
    }

    public PageVo<TestValidListVO> adaptList(TestValidListReq req) {
        return queryData(req, Purchase::getAdaptStatus, "actual_adaption_time");
    }

    public PageVo<TestValidListVO> selfTestList(TestValidListReq req) {
        return queryData(req, Purchase::getSelfTestStatus, "actual_self_test_time");
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitTestValidInfo(TestValidInfoReq req) {
        List<Purchase> purchases = purchaseService.listByIds(req.getBidIds());
        purchases.forEach(purchase -> {
            if (!TenderTestValidStatus.WITHOUT_TEST_VALID_INFO.eq(purchase.getTestValidStatus())
                    && !TenderTestValidStatus.TEST_VALID_INFO_FAILED.eq(purchase.getTestValidStatus())) {
                throw BizException.wrap("该状态下不支持提交测试验证信息");
            }
        });
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        ProjectProcessType instType = ProjectProcessType.TEST_VALID;
        WflowModels model = processModelManage.getWflowModelsNoNull(instType, user.getMhUnitId());
        LocalDateTime now = LocalDateTime.now();
        for (Purchase purchase : purchases) {
            Long projectId = purchase.getProjectId();
            Project project = projectService.getNewProject(projectId);
            ProcessStartParamsVo instParam = new ProcessStartParamsVo();
            instParam.setUser(declaredProjectManage.buildUser(user));
            instParam.setProcessUsers(Collections.emptyMap());
            instParam.setFormData(Collections.emptyMap());
            // 获取发起单位、发起单位主管单位、发起单位上级条线主管单位信息
            Map<String, OrgInfoDTO> orgMap = declaredProjectManage.buildOrgModelMap(user, project);
            String instanceId = processInstanceService.startProcessLs(model, instParam, orgMap);
            testValidStateMachineUtil.pass(purchase);
            purchaseService.updateById(purchase);
            Operation operation = operationService.getByBidId(purchase.getId());
            if (operation == null) {
                operation = new Operation();
                operation.setBidId(purchase.getId());
                operation.setCreateBy(user.getUsername());
                operation.setCreateOn(now);
                operation.setProjectId(projectId);
                operation.setProjectCode(project.getProjectCode());
            }
            operation.setUpdateBy(user.getUsername());
            operation.setUpdateOn(now);
            operation.setActualTestValidTime(req.getActualTime());
            operation.setTestValidFiles(req.getFiles());
            operation.setTestValidRemark(req.getRemark());
            operationService.saveOrUpdate(operation);
            saveProjectPurchaseInstCode(purchase.getId(), instanceId, project.getId(), instType);
        }
    }

    private void saveProjectPurchaseInstCode(Long bidId,
                                             String instanceId,
                                             Long projectId,
                                             ProjectProcessType instType) {
        // 保存标段实列关系
        PurchaseInst purchaseInst = new PurchaseInst();
        purchaseInst.setBidId(bidId);
        purchaseInst.setInstCode(instanceId);
        purchaseInst.setProjectId(projectId);
        purchaseInst.setInstType(instType);
        purchaseInstService.save(purchaseInst);
        // 保存项目实列关系
        ProjectInst projectInst = new ProjectInst();
        projectInst.setInstType(instType);
        projectInst.setInstCode(instanceId);
        projectInst.setProjectId(projectId);
        projectInstService.save(projectInst);
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitSelfTestInfo(TestValidInfoReq req) {
        List<Purchase> purchases = purchaseService.listByIds(req.getBidIds());
        for (Purchase purchase : purchases) {
            if (!TenderSelfTestStatus.WITHOUT_SELF_TEST_INFO.eq(purchase.getSelfTestStatus())
                    && !TenderSelfTestStatus.SELF_TEST_INFO_FAILED.eq(purchase.getSelfTestStatus())) {
                throw BizException.wrap("该状态下不支持提交系统自测信息");
            }
        }
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        ProjectProcessType instType = ProjectProcessType.SELF_TEST;
        WflowModels model = processModelManage.getWflowModelsNoNull(instType, user.getMhUnitId());

        LocalDateTime now = LocalDateTime.now();
        for (Purchase purchase : purchases) {
            Long projectId = purchase.getProjectId();
            Project project = projectService.getNewProject(projectId);
            ProcessStartParamsVo instParam = new ProcessStartParamsVo();
            instParam.setUser(declaredProjectManage.buildUser(user));
            instParam.setProcessUsers(Collections.emptyMap());
            instParam.setFormData(Collections.emptyMap());
            // 获取发起单位、发起单位主管单位、发起单位上级条线主管单位信息
            Map<String, OrgInfoDTO> orgMap = declaredProjectManage.buildOrgModelMap(user, project);
            String instanceId = processInstanceService.startProcessLs(model, instParam, orgMap);
            selfTestStateMachineUtil.pass(purchase);
            purchaseService.updateById(purchase);
            Operation operation = operationService.getByBidId(purchase.getId());
            if (operation == null) {
                operation = new Operation();
                operation.setBidId(purchase.getId());
                operation.setCreateBy(user.getUsername());
                operation.setCreateOn(now);
                operation.setProjectId(projectId);
                operation.setProjectCode(project.getProjectCode());
            }
            operation.setActualSelfTestTime(req.getActualTime());
            operation.setSelfTestFiles(req.getFiles());
            operation.setSelfTestRemark(req.getRemark());
            operation.setUpdateBy(user.getUsername());
            operation.setUpdateOn(now);
            operationService.saveOrUpdate(operation);
            saveProjectPurchaseInstCode(purchase.getId(), instanceId, project.getId(), instType);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitAdaptionInfo(TestValidInfoReq req) {
        List<Purchase> purchases = purchaseService.listByIds(req.getBidIds());
        for (Purchase purchase : purchases) {
            if (!TenderAdaptStatus.WITHOUT_ADAPT_INFO.eq(purchase.getAdaptStatus())
                    && !TenderAdaptStatus.ADAPT_INFO_FAILED.eq(purchase.getAdaptStatus())) {
                throw BizException.wrap("该状态下不支持提交适配改造信息");
            }
        }
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        ProjectProcessType instType = ProjectProcessType.ADAPTION;
        WflowModels model = processModelManage.getWflowModelsNoNull(instType, user.getMhUnitId());

        LocalDateTime now = LocalDateTime.now();
        for (Purchase purchase : purchases) {
            Long projectId = purchase.getProjectId();
            Project project = projectService.getNewProject(projectId);
            ProcessStartParamsVo instParam = new ProcessStartParamsVo();
            instParam.setUser(declaredProjectManage.buildUser(user));
            instParam.setProcessUsers(Collections.emptyMap());
            instParam.setFormData(Collections.emptyMap());
            // 获取发起单位、发起单位主管单位、发起单位上级条线主管单位信息
            Map<String, OrgInfoDTO> orgMap = declaredProjectManage.buildOrgModelMap(user, project);
            String instanceId = processInstanceService.startProcessLs(model, instParam, orgMap);
            adaptStateMachineUtil.pass(purchase);
            purchaseService.updateById(purchase);
            Operation operation = operationService.getByBidId(purchase.getId());
            if (operation == null) {
                operation = new Operation();
                operation.setBidId(purchase.getId());
                operation.setCreateBy(user.getUsername());
                operation.setCreateOn(now);
                operation.setProjectId(projectId);
                operation.setProjectCode(project.getProjectCode());
            }
            operation.setActualAdaptionTime(req.getActualTime());
            operation.setAdaptionFiles(req.getFiles());
            operation.setAdaptionRemark(req.getRemark());
            operation.setUpdateBy(user.getUsername());
            operation.setUpdateOn(now);
            operationService.saveOrUpdate(operation);
            saveProjectPurchaseInstCode(purchase.getId(), instanceId, project.getId(), instType);
        }
    }

    public PageVo<XcfhxApplyListVO> pageXcfhxApply(XcfhxListReq req) {
        Integer pageNumber = req.getPageNumber();
        Integer pageSize = req.getPageSize();
        //手动添加的
        List<XcfhxApplyListVO> manuals = xinchuangService.listXcfhxApplyListVO(req);
        //系统和大数据推送的
        List<XcfhxApplyListVO> others = purchaseService.listXcfhxApplyListVO(req);

        //合并
        List<XcfhxApplyListVO> merge = Stream.of(manuals, others)
                .flatMap(List::stream)
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
        return PageVo.of(merge.stream().sorted(Comparator.comparing(XcfhxApplyListVO::getCreateOn).reversed())
                .skip((long) (pageNumber - 1) * pageSize)
                .limit(pageSize).collect(Collectors.toList()), merge.size());
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    public XinchuangVO detailXcfhxApply(Long id) {
        Xinchuang xinchuang = xinchuangService.getById(id);
        if (Objects.isNull(xinchuang)) {
            return null;
        }
        return BeanUtil.copyProperties(xinchuang, XinchuangVO.class);
    }

    public List<PurchaseNoticeTenderDTO> listPurchaseIntention(Long unitId) {
        List<MhPurchaseIntentionDTO> intentions = mhApiClient.listPurchaseNotice(unitId);
        if (intentions.isEmpty()) {
            return Collections.emptyList();
        }
        return intentions.stream().map(w -> {
            PurchaseScopeEnum scope = PurchaseScopeEnum.getByMhCode(w.getBuyScope())
                    .orElseThrow(() -> BizException.wrap("采购范围无效"));
            PurchaseNoticeTenderDTO item = new PurchaseNoticeTenderDTO();
            item.setBidName(w.getBuyItem());
            item.setBidBudget(BigDecimal.valueOf(w.getBudgetMoney())
                    .divide(BigDecimal.valueOf(10000), RoundingMode.HALF_UP));
            item.setPurchaseMethod(w.getBuyWay());
            item.setPurchaseContact(w.getLinkUser());
            item.setPurchasePhone(w.getLinkPhone());
            item.setPurchaseConnectAddress(w.getDetailAddress());
            BizUtils.notNull(w.getBidOpenTime(), t -> item.setBidStartTime(LocalDateTimeUtil.of(t)));
            BizUtils.notNull(w.getEndTime(), t -> item.setBidEndTime(LocalDateTimeUtil.of(t)));
            item.setPurchaseRemark(w.getBuyDesc());
            item.setPurchaseScope(scope.getCode());
            return item;
        }).collect(Collectors.toList());
    }

    /**
     * 新增信创系统
     *
     * @param dto
     * @return
     */
    public Long addSystem(XcfhxApplySaveDTO dto) {
        Xinchuang xinchuang = BeanUtil.copyProperties(dto, Xinchuang.class);
        xinchuang.setSourceType(PurchaseSourceTypeEnum.MANUAL_ADDITION.getCode());
        if (Boolean.TRUE.equals(dto.getMatchXcfhx())) {
            if (StrUtils.isNotBlank(dto.getMhXcfhxReportFile())) {
                String fileName = xinchuang.getBidName() + "-信创符合性测评报告.";
                xinchuang.setXcfhxReportFiles(mhXchxFileHelper.getXchxFile(dto.getMhXcfhxReportFile(), fileName));
            }
        }
        if (xinchuangService.save(xinchuang)) {
            return xinchuang.getId();
        }
        throw BizException.wrap("新增失败");
    }

    public Long updateSystem(XcfhxApplyEditDTO dto) {
        Xinchuang old = xinchuangService.getById(dto.getId());
        VUtils.isTrue(Objects.isNull(old)).throwMessage("数据不存在");
        Xinchuang xinchuang = BeanUtil.copyProperties(dto, Xinchuang.class);
        if (Boolean.TRUE.equals(dto.getMatchXcfhx()) && StrUtils.isNotBlank(dto.getMhXcfhxReportFile()) &&
                !dto.getMhXcfhxReportFile().equals(old.getMhXcfhxReportFile())) {
            String fileName = xinchuang.getBidName() + "-信创符合性测评报告.";
            xinchuang.setXcfhxReportFiles(mhXchxFileHelper.getXchxFile(dto.getMhXcfhxReportFile(), fileName));
        }
        if (xinchuangService.updateById(xinchuang)) {
            return xinchuang.getId();
        }
        throw BizException.wrap("编辑失败");
    }

    public String xcfhxReportFiles(String code) {
        return mhXchxFileHelper.getXchxFile(code, null);
    }

    public void exportXcfhFile(String code, HttpServletResponse response) throws UnsupportedEncodingException {
        mhXchxFileHelper.exportReport(code, response);
    }
}
