package com.hz.pm.api.projectdeclared.model.entity;

import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * <p>
 * 采购信息（标段）
 * </p>
 *
 * @author WendyYang
 * @since 15:07 2024/2/28
 */
@Data
@TableName("nd_purchase")
@ApiModel(value = "采购对象")
public class Purchase {

    @TableId(type = IdType.AUTO)
    @ApiModelProperty("id")
    private Long id;

    @ApiModelProperty("标段名称")
    private String bidName;

    @ApiModelProperty("项目ID")
    private Long projectId;

    @ApiModelProperty("创建时间")
    @TableField(fill = FieldFill.INSERT)
    private LocalDateTime createOn;

    @ApiModelProperty("修改时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private LocalDateTime updateOn;

    @ApiModelProperty("创建人 工号")
    @TableField(fill = FieldFill.INSERT)
    private String createBy;

    @ApiModelProperty("修改人 工号")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private String updateBy;

    @ApiModelProperty("供应商")
    private String supplier;

    @ApiModelProperty("供应商社会信用代码")
    private String supplierSocialCreditCode;

    @ApiModelProperty("采购方式 1公开招标 2自行采购")
    private String purchaseMethod;

    @ApiModelProperty("成交金额")
    private BigDecimal transactionAmount;

    @ApiModelProperty("成交时间")
    private LocalDateTime transactionTime;

    @ApiModelProperty("代理机构")
    private String agency;

    @ApiModelProperty("标段类型")
    private Integer bidType;

    @ApiModelProperty("投标文件")
    private String biddingDoc;

    @ApiModelProperty("中标通知书")
    private String acceptanceLetter;

    @ApiModelProperty("标段建设内容")
    private String bidBuildContent;

    @ApiModelProperty("代理机构社会统一信用代码")
    private String purchaseSocialCreditCode;

    @ApiModelProperty("招标文件")
    private String bidDoc;

    @ApiModelProperty("预算执行确认书编号")
    private String budgetExecConfirmNo;

    @ApiModelProperty("投标地址")
    private String bidAddress;

    @ApiModelProperty("标项预算")
    private BigDecimal bidBudget;

    @ApiModelProperty("开标时间")
    private LocalDateTime bidStartTime;

    @ApiModelProperty("投标截止时间")
    private LocalDateTime bidEndTime;

    private Integer status;

    @ApiModelProperty("采购范围：1 公开采购、2 非公开采购")
    private Integer purchaseScope;

    @ApiModelProperty("采购单位")
    private String purchaseUnit;

    @ApiModelProperty("采购单位ID")
    private Long purchaseUnitId;

    @ApiModelProperty("采购联系人")
    private String purchaseContact;

    @ApiModelProperty("采购联系电话")
    private String purchasePhone;

    @ApiModelProperty("采购联系地址")
    private String purchaseConnectAddress;

    @ApiModelProperty("采购说明")
    private String purchaseRemark;

    @ApiModelProperty("初验材料")
    private String preliminaryInspectionMaterials;

    @ApiModelProperty("（初验）是否完成日志数据归集")
    private Boolean isCompletedLogCollection;

    @ApiModelProperty("（初验）实际成效指标")
    private String actualPerformanceIndicators;

    @ApiModelProperty("终验申报材料")
    private String finalInspectionMaterials;

    @ApiModelProperty("是否符合信创符合性要求")
    private Boolean matchXcfhx;

    @ApiModelProperty("信创符合性申请备注")
    private String xcfhxApplyRemark;

    @ApiModelProperty("信创符合性申请其他附件")
    private String xcfhxApplyFiles;

    @ApiModelProperty("信创符合性测评报告文件")
    private String mhXcfhxReportFile;

    @ApiModelProperty("信创符合性测评记录ID")
    private String mhXcfhxReportRecordId;

    @ApiModelProperty("信创报告")
    private String xcfhxReportFiles;

    @ApiModelProperty("适配改造状态")
    private Integer adaptStatus;

    @ApiModelProperty("系统自测状态")
    private Integer selfTestStatus;

    @ApiModelProperty("测试验证状态")
    private Integer testValidStatus;

    @ApiModelProperty("信创符合性申请状态")
    private Integer xcfhxApplyStatus;

    @ApiModelProperty("资源类型 1手动新增 2系统生成 3大数据局")
    private Integer sourceType;

    @ApiModelProperty("项目名称")
    private String projectName;

}
