package com.hz.pm.api.projectdeclared.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.IService;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.datascope.model.DataScopeDTO;
import com.hz.pm.api.datascope.utils.DataScopeUtil;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.req.XcfhxListReq;
import com.hz.pm.api.projectdeclared.model.vo.XcfhxApplyListVO;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderStatus;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.ningdatech.basic.util.CollUtils;
import lombok.extern.slf4j.Slf4j;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author zpf
 * @since 2023-05-29
 */
public interface IPurchaseService extends IService<Purchase> {

    default List<Purchase> listByProjectId(Long projectId) {
        return list(Wrappers.lambdaQuery(Purchase.class)
                .eq(Purchase::getProjectId, projectId)
                .orderByAsc(Purchase::getId));
    }

    default List<Purchase> listByProjectIds(Collection<Long> projectIds) {
        return list(Wrappers.lambdaQuery(Purchase.class)
                .in(Purchase::getProjectId, projectIds)
                .orderByAsc(Purchase::getId));
    }

    default Set<Long> listProjectIdByStatus(Collection<TenderStatus> status) {
        if (CollUtil.isEmpty(status)) {
            return Collections.emptySet();
        }
        List<Integer> statusCodes = status.stream()
                .map(TenderStatus::getCode)
                .collect(Collectors.toList());
        LambdaQueryWrapper<Purchase> query = Wrappers.lambdaQuery(Purchase.class)
                .select(Purchase::getProjectId)
                .in(Purchase::getStatus, statusCodes);
        return CollUtils.fieldSet(list(query), Purchase::getProjectId);
    }

    boolean buildPurchaseQueryPermission(LambdaQueryWrapper<Purchase> query, UserFullInfoDTO user);

    List<XcfhxApplyListVO> listXcfhxApplyListVO(XcfhxListReq req);
}
