package com.hz.pm.api.workbench.manage;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.model.entity.KeyValDTO;
import com.hz.pm.api.common.statemachine.event.*;
import com.hz.pm.api.common.util.DecimalUtil;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.entity.ProjectGovSystemReplaceInfos;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.handle.AbstractProcessHandle.ProcessNode;
import com.hz.pm.api.projectlib.manage.ProjectLibManage;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.GovSystemReplaceTypeEnum;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.service.IProjectGovSystemReplaceInfosService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import com.hz.pm.api.sys.manage.EarlyWarningManage;
import com.hz.pm.api.sys.model.entity.WflowEarlyWarningRecords;
import com.hz.pm.api.sys.service.IEarlyWarningRecordsService;
import com.hz.pm.api.todocenter.manage.TodoCenterManage;
import com.hz.pm.api.todocenter.model.req.ToBeProcessedReq;
import com.hz.pm.api.todocenter.model.vo.ResToBeProcessedVO;
import com.hz.pm.api.todocenter.model.vo.TodoCenterStatisticsVO;
import com.hz.pm.api.todocenter.model.vo.TodoVO;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.hz.pm.api.workbench.converter.WorkbenchConverter;
import com.hz.pm.api.workbench.model.WorkbenchProjectLibReq;
import com.hz.pm.api.workbench.model.WorkbenchReq;
import com.hz.pm.api.workbench.model.vo.ProjectProcessStageStatisticsVO;
import com.hz.pm.api.workbench.model.vo.ProjectTotalViewVO;
import com.hz.pm.api.workbench.model.vo.WorkbenchVO;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.StrPool;
import com.wflow.bean.entity.WflowEarlyWarning;
import com.wflow.service.IEarlyWarningService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.function.ToLongBiFunction;
import java.util.function.ToLongFunction;
import java.util.stream.Collectors;

/**
 * <p>
 * WorkbenchManage
 * </p>
 *
 * @author WendyYang
 * @since 18:58 2024/3/23
 */
@Slf4j
@Component
@AllArgsConstructor
public class WorkbenchManage {

    private final TodoCenterManage todoCenterManage;
    private final IEarlyWarningRecordsService earlyWarningRecordsService;
    private final IEarlyWarningService earlyWarningService;
    private final UserInfoHelper userInfoHelper;
    private final ProjectLibManage projectLibManage;
    private final EarlyWarningManage earlyWarningManage;
    private final IProjectService projectService;
    private final IProjectStatusChangeService projectStatusChangeService;
    private final IPurchaseService purchaseService;
    private final MhUnitCache mhUnitCache;
    private final IProjectGovSystemReplaceInfosService systemReplaceInfosService;
    private final IPurchaseStatusChangeService purchaseStatusChangeService;

    public WorkbenchVO getWorkbenchData(Integer year) {
        WorkbenchVO res = new WorkbenchVO();
        Long userId = LoginUserUtil.getUserId();
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(userId);
        // 待办中心数据
        TodoCenterStatisticsVO todoStat = todoCenterManage.todoCenterStatistics(userId);
        ToBeProcessedReq todoReq = new ToBeProcessedReq();
        todoReq.page(1, 50);
        todoReq.setLoginUserId(userId);
        todoReq.setUserId(userId);
        PageVo<ResToBeProcessedVO> todoPage = todoCenterManage.todoProjectList(todoReq);
        List<TodoVO> todoList = CollUtils.fieldList(todoPage.getRecords(), WorkbenchConverter::convert);
        todoStat.setTodoList(todoList);
        res.setTodoCerter(todoStat);
        // 项目列表
        ProjectListReq projReq = new ProjectListReq();
        projReq.page(1, 20);
        projReq.setProjectYear(year);
        PageVo<ProjectLibListItemVO> projPage = projectLibManage.listWithPermissionWorkBench(projReq, user);
        res.setProjects(new ArrayList<>(projPage.getRecords()));
        return res;
    }

    /**
     * 催办
     *
     * @param recordId \
     * @return \
     */
    public synchronized String urging(Long recordId) {
        WflowEarlyWarningRecords warningRecords = earlyWarningRecordsService.getById(recordId);
        Assert.notNull(warningRecords, "催办失败,id不存在!");
        String projectCode = warningRecords.getProjectCode();
        Project project = projectService.getNoNull(projectCode);
        String batchEmployees = warningRecords.getBatchUserIds();
        Integer ruleType = warningRecords.getRuleType();
        String areaCode = warningRecords.getAreaCode();
        //如果是临期 可以取到 超期的小时数
        String path = warningRecords.getPath();
        LocalDateTime instStart = warningRecords.getInstStart();
        Integer biz = warningRecords.getBiz();
        if (StringUtils.isBlank(batchEmployees)) {
            //如果批次员工号不存在 就取当前
            batchEmployees = warningRecords.getWarningUserId();
        }
        Assert.notEmpty(batchEmployees, "催办失败,员工为空!");
        String[] employees = batchEmployees.split(StrPool.COMMA);
        String noticeContent = warningRecords.getNoticeContent();
        //查出配置的模板
        WflowEarlyWarning config = earlyWarningService.getOne(Wrappers.lambdaQuery(WflowEarlyWarning.class)
                .eq(WflowEarlyWarning::getAreaCode, areaCode)
                .eq(WflowEarlyWarning::getRuleType, ruleType)
                .last(BizConst.LIMIT_1));
        //如果配置能取到 就取模板的
        if (Objects.nonNull(config)) {
            noticeContent = config.getNoticeContent();
        }

        //算出 当前和他的 小时差 临期
        long adventBetween = ChronoUnit.HOURS.between(instStart, LocalDateTime.now());
        String noticeMethod = warningRecords.getNoticeMethod();
        Integer noticeType = warningRecords.getNoticeType();
        for (String employee : employees) {
            earlyWarningManage.urging(noticeMethod, noticeContent,
                    (int) adventBetween, (int) adventBetween,
                    biz, instStart, Long.valueOf(employee),
                    project, ruleType, noticeType, path,
                    batchEmployees, warningRecords.getNodeId());
        }
        return "催办成功了" + employees.length + "个人";
    }

    private void initWorkbenchDefaultParam(WorkbenchReq req) {
        if (req.getUnitId() == null) {
            req.setUnitId(LoginUserUtil.loginUserDetail().getMhUnitId());
        }
        if (req.getProjectYear() == null) {
            req.setProjectYear(LocalDate.now().getYear());
        }
    }

    public ProjectTotalViewVO projectTotalView(WorkbenchReq req) {
        ProjectTotalViewVO retDetail = new ProjectTotalViewVO();
        initWorkbenchDefaultParam(req);
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .select(Project::getReviewAmount, Project::getApprovalGovOwnFinanceAmount, Project::getId)
                .eq(Project::getProjectYear, req.getProjectYear())
                .eq(Project::getNewest, Boolean.TRUE);
        List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(req.getUnitId());
        query.in(Project::getBuildOrgCode, CollUtils.convert(viewUnitIds, String::valueOf));
        List<Project> projects = projectService.list(query);
        if (projects.isEmpty()) {
            return retDetail;
        }
        BigDecimal approvalAmount = DecimalUtil.sum(projects, Project::getApprovalGovOwnFinanceAmount);
        BigDecimal reviewAmount = DecimalUtil.sum(projects, Project::getReviewAmount);
        retDetail.setTotalExecuteAmount(BigDecimal.ZERO);
        retDetail.setTotalApprovalAmount(approvalAmount);
        retDetail.setTotalReviewAmount(reviewAmount);
        Wrapper<ProjectGovSystemReplaceInfos> pgsriQuery = Wrappers.lambdaQuery(ProjectGovSystemReplaceInfos.class)
                .in(ProjectGovSystemReplaceInfos::getProjectId,
                        CollUtils.fieldList(projects, Project::getId));
        List<ProjectGovSystemReplaceInfos> systemReplaceInfos = systemReplaceInfosService.list(pgsriQuery);
        Map<String, Long> typeCountMap = systemReplaceInfos.stream()
                .map(w -> StrUtil.split(w.getReplaceType(), ","))
                .flatMap(Collection::stream)
                .collect(Collectors.groupingBy(w -> {
                    if (!NumberUtil.isNumber(w)) {
                        return "其他";
                    } else {
                        return GovSystemReplaceTypeEnum.get(Integer.parseInt(w))
                                .flatMap(t -> Optional.of(t.getVal()))
                                .orElse("其他");
                    }
                }, Collectors.counting()));
        List<KeyValDTO<String, Integer>> systemTypes = typeCountMap.entrySet().stream()
                .map(w -> KeyValDTO.of(w.getKey(), w.getValue().intValue()))
                .collect(Collectors.toList());
        retDetail.setSystemTypes(systemTypes);
        return retDetail;
    }

    public long getProjectCount(List<Long> projectIds, ProjectStateChangeEvent event) {
        return projectStatusChangeService.count(Wrappers.lambdaQuery(ProjectStatusChange.class)
                .in(ProjectStatusChange::getProjectId, projectIds)
                .eq(ProjectStatusChange::getEvent, event));
    }

    public List<ProjectProcessStageStatisticsVO> projectProcessStageStatistics(WorkbenchReq req) {
        initWorkbenchDefaultParam(req);
        List<ProjectProcessStageStatisticsVO> retData = new ArrayList<>();
        List<Long> allVersionProjectIds = new ArrayList<>();
        Map<String, Map<Long, Long>> purchaseEventMap = new HashMap<>();
        Map<Long, Long> projectPurchaseCountMap = new HashMap<>();
        ToLongFunction<AbstractStateChangeEvent> countPurchase = event -> {
            Map<Long, Long> projBizMap = purchaseEventMap.getOrDefault(event.name(), Collections.emptyMap());
            return projBizMap.values().stream().mapToLong(l -> l).sum();
        };
        ToLongBiFunction<AbstractStateChangeEvent, ProcessNode> countProject = (event, node) -> {
            Map<Long, Long> projBizMap = purchaseEventMap.getOrDefault(event.name(), Collections.emptyMap());
            if (node.equals(ProcessNode.PROJECT_ADAPTION)
                    || node.equals(ProcessNode.SYSTEM_SELF_TEST)
                    || node.equals(ProcessNode.SYSTEM_TEST_VALID)
                    || node.equals(ProcessNode.XCFHX_REVIEW)) {
                return projBizMap.size();
            } else {
                return projectPurchaseCountMap.entrySet().stream().filter(w -> {
                    Long finishedBizCount = projBizMap.get(w.getKey());
                    return finishedBizCount != null && finishedBizCount.equals(w.getValue());
                }).count();
            }
        };
        // 查询完成立项备案的项目
        List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(req.getUnitId());
        Wrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .select(Project::getId, Project::getApprovalAmount, Project::getProjectCode)
                .eq(Project::getProjectYear, req.getProjectYear())
                .eq(Project::getNewest, Boolean.TRUE)
                .exists("select 1 from nd_project_status_change npsc " +
                        "where npsc.project_code = nd_project.project_code and " +
                        "npsc.event = {0}", ProjectStateChangeEvent.COMPLIANCE_REVIEW_PASS)
                .in(Project::getBuildOrgCode, CollUtils.convert(viewUnitIds, String::valueOf));
        List<Project> projects = projectService.list(query);
        for (ProcessNode node : ProcessNode.ALL) {
            long projectCount = 0;
            long bidCount = 0;
            if (!projects.isEmpty()) {
                switch (node) {
                    case PROJECT_DECLARED:
                        projectCount = projects.size();
                        for (Project project : projects) {
                            allVersionProjectIds.addAll(ProjectIdCodeCacheUtil.get(project.getProjectCode()));
                        }
                        break;
                    case PROJECT_REVIEW:
                        projectCount = getProjectCount(allVersionProjectIds, ProjectStateChangeEvent.PROJECT_REVIEW_PASS);
                        break;
                    case APPROVAL_AMOUNT:
                        projectCount = projects.stream()
                                .filter(w -> w.getApprovalAmount() != null)
                                .filter(w -> w.getApprovalAmount().compareTo(BigDecimal.ZERO) > 0)
                                .count();
                        break;
                    case PROJECT_APPROVAL:
                        projectCount = getProjectCount(allVersionProjectIds, ProjectStateChangeEvent.DECLARED_RECORD_PASS);
                        break;
                    case PROJECT_PURCHASE:
                        List<Purchase> purchases = purchaseService.list(Wrappers.lambdaQuery(Purchase.class)
                                .select(Purchase::getId, Purchase::getProjectId)
                                .in(Purchase::getProjectId, allVersionProjectIds));
                        bidCount = purchases.size();
                        projectPurchaseCountMap.putAll(CollUtils.groupCount(purchases, Purchase::getProjectId));
                        projectCount = projectPurchaseCountMap.size();
                        if (bidCount > 0) {
                            Wrapper<PurchaseStatusChange> pQuery = Wrappers.lambdaQuery(PurchaseStatusChange.class)
                                    .select(PurchaseStatusChange::getEvent,PurchaseStatusChange::getProjectId)
                                    .in(PurchaseStatusChange::getProjectId, allVersionProjectIds)
                                    .in(PurchaseStatusChange::getEvent, AdaptStateChangeEvent.ADAPT_INFO_PASSED,
                                            SelfTestStateChangeEvent.SELF_TEST_PASSED,
                                            TestValidStateChangeEvent.TEST_VALID_INFO_PASSED,
                                            TenderStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES,
                                            XcfhxStateChangeEvent.XCFHX_APPLY_PASSED,
                                            TenderStateChangeEvent.FINALLY_INSPECTED_PASSED);
                            List<PurchaseStatusChange> changes = purchaseStatusChangeService.list(pQuery);
                            Map<String, Map<Long, Long>> purchaseEventMapTmp = changes.stream()
                                    .collect(Collectors.groupingBy(PurchaseStatusChange::getEvent,
                                            Collectors.collectingAndThen(Collectors.toList(),
                                                    w -> CollUtils.groupCount(w, PurchaseStatusChange::getProjectId))));
                            purchaseEventMap.putAll(purchaseEventMapTmp);
                        }
                        break;
                    case PROJECT_ADAPTION:
                        bidCount = countPurchase.applyAsLong(AdaptStateChangeEvent.ADAPT_INFO_PASSED);
                        projectCount = countProject.applyAsLong(AdaptStateChangeEvent.ADAPT_INFO_PASSED, node);
                        break;
                    case SYSTEM_SELF_TEST:
                        bidCount = countPurchase.applyAsLong(SelfTestStateChangeEvent.SELF_TEST_PASSED);
                        projectCount = countProject.applyAsLong(SelfTestStateChangeEvent.SELF_TEST_PASSED, node);
                        break;
                    case SYSTEM_TEST_VALID:
                        bidCount = countPurchase.applyAsLong(TestValidStateChangeEvent.TEST_VALID_INFO_PASSED);
                        projectCount = countProject.applyAsLong(TestValidStateChangeEvent.TEST_VALID_INFO_PASSED, node);
                        break;
                    case FIRST_INSPECTED:
                    case PILOT_RUNNING:
                        // 初验试运行数量同步
                        bidCount = countPurchase.applyAsLong(TenderStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES);
                        projectCount = countProject.applyAsLong(TenderStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES, node);
                        break;
                    case XCFHX_REVIEW:
                        bidCount = countPurchase.applyAsLong(XcfhxStateChangeEvent.XCFHX_APPLY_PASSED);
                        projectCount = countProject.applyAsLong(XcfhxStateChangeEvent.XCFHX_APPLY_PASSED, node);
                        break;
                    case FINAL_INSPECTED:
                        bidCount = countPurchase.applyAsLong(TenderStateChangeEvent.FINALLY_INSPECTED_PASSED);
                        projectCount = countProject.applyAsLong(TenderStateChangeEvent.FINALLY_INSPECTED_PASSED, node);
                        break;
                    default:
                        break;
                }
                retData.add(ProjectProcessStageStatisticsVO.builder()
                        .stage(node)
                        .stageName(node.getName())
                        .projectCount((int) projectCount)
                        .bidCount((int) bidCount)
                        .build());
            }
        }
        return retData;
    }

    public PageVo<ProjectLibListItemVO> pageProjectLib(WorkbenchProjectLibReq req) {
        Assert.notNull(req.getProcessNode(), "节点不能为空");
        Long unitId;
        if (req.getUnitId() != null) {
            unitId = req.getUnitId();
        } else {
            unitId = LoginUserUtil.loginUserDetail().getMhUnitId();
        }
        List<Long> viewUnitIds = mhUnitCache.getViewChildIdsRecursion(unitId);
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .eq(req.getProjectYear() != null, Project::getProjectYear, req.getProjectYear())
                .like(StrUtil.isNotBlank(req.getBuildOrg()), Project::getBuildOrgName, req.getBuildOrg())
                .like(StrUtil.isNotBlank(req.getProjectName()), Project::getProjectName, req.getProjectName())
                .in(Project::getBuildOrgCode, CollUtils.convert(viewUnitIds, String::valueOf))
                // 评审金额
                .ge(req.getReviewAmountMin() != null, Project::getReviewAmount, req.getReviewAmountMin())
                .le(req.getReviewAmountMax() != null, Project::getReviewAmount, req.getReviewAmountMax())
                // 下达金额
                .ge(req.getApproveAmountMin() != null, Project::getApprovalAmount, req.getApproveAmountMin())
                .ge(req.getApproveAmountMax() != null, Project::getApprovalAmount, req.getApproveAmountMax())
                .eq(Project::getNewest, Boolean.TRUE)
                .orderByDesc(Project::getUpdateOn);
        switch (req.getProcessNode()) {
            case PROJECT_DECLARED:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.COMPLIANCE_REVIEW_PASS);
                break;
            case PROJECT_REVIEW:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.PROJECT_REVIEW_PASS);
                break;
            case APPROVAL_AMOUNT:
                query.isNotNull(Project::getApprovalAmount).gt(Project::getApprovalAmount, BigDecimal.ZERO);
                break;
            case PROJECT_APPROVAL:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.DECLARED_RECORD_PASS);
                break;
            case PROJECT_PURCHASE:
                projectQueryByProjectStatusChange(query, ProjectStateChangeEvent.SUBMIT_PURCHASE_NOTICE);
                break;
            case PROJECT_ADAPTION:
                projectQueryByPurchaseStatusChange(query, AdaptStateChangeEvent.ADAPT_INFO_PASSED);
                break;
            case SYSTEM_SELF_TEST:
                projectQueryByPurchaseStatusChange(query, SelfTestStateChangeEvent.SELF_TEST_PASSED);
                break;
            case SYSTEM_TEST_VALID:
                projectQueryByPurchaseStatusChange(query, TestValidStateChangeEvent.TEST_VALID_INFO_PASSED);
                break;
            case FIRST_INSPECTED:
            case PILOT_RUNNING:
                projectQueryByPurchaseStatusChange(query, TenderStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES);
                break;
            case XCFHX_REVIEW:
                projectQueryByPurchaseStatusChange(query, XcfhxStateChangeEvent.XCFHX_APPLY_PASSED);
                break;
            case FINAL_INSPECTED:
                projectQueryByPurchaseStatusChange(query, TenderStateChangeEvent.FINALLY_INSPECTED_PASSED);
                break;
            default:
                return PageVo.empty();
        }
        Page<Project> page = projectService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setProjectName(w.getProjectName());
            // 项目申报
            item.setFromType("1");
            item.setProjectCode(w.getProjectCode());
            item.setArea(w.getArea());
            item.setAreaCode(w.getAreaCode());
            item.setCreateOn(w.getCreateOn());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectType(w.getProjectType());
            item.setProjectYear(w.getProjectYear());
            item.setBuildOrg(w.getBuildOrgName());
            item.setBizDomain(w.getBizDomain());
            item.setProcessStatus(w.getProcessStatus());
            item.setInstCode(w.getInstCode());
            item.setIsHigherSuperOrg(w.getIsHigherSuperOrg());
            item.setApprovedAmount(w.getApprovalAmount());
            item.setReviewAmount(w.getReviewAmount());
            item.setPrePlanProjectId(w.getPrePlanProjectId());
            return item;
        });
        return PageVo.of(records, page.getTotal());
    }

    private static void projectQueryByProjectStatusChange
            (LambdaQueryWrapper<Project> query, AbstractStateChangeEvent event) {
        query.exists("select 1 from nd_project_status_change npsc" +
                " where nd_project.project_code = npsc.project_code" +
                " and npsc.event = {0}", event);
    }

    private static void projectQueryByPurchaseStatusChange
            (LambdaQueryWrapper<Project> query, AbstractStateChangeEvent event) {
        query.exists("select 1 from nd_purchase_status_change npsc" +
                " where nd_project.project_code = npsc.project_code" +
                " and npsc.event ={0}", event.name());
    }

}
